/*
    ir_decoder.c - 
    Copyright (C) 2008  <telekatz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "lcd.h"
#include "fonty.h"
#include "keyboard.h"
#include "lpc2220.h"
#include "irq.h"
#include "backlight.h"
#include "ir_raw.h"
#include "ir_decoder.h"
#include "global.h"

extern volatile unsigned int keys[2];
extern volatile unsigned char autorepeat;
extern void setSpeed(unsigned char sp);

struct widetable_ {
	unsigned char count;	
	unsigned int value[15];
};

unsigned long waitIrHi(unsigned int* onpulse, unsigned long* time, unsigned long start) {
	
	unsigned long lowpulse;
	unsigned long l;
	unsigned long x;	
	
	*onpulse = 0;
	lowpulse = 0;
	while (1) {
		T0CCR = (1<<9) | (1<<11);
		while (!(T0IR));
		T0IR = 0xff;
		T0CCR = (1<<9) | (1<<10) | (1<<11);
		l = T0CR3;
		*time = l-start + lowpulse;
		(*onpulse)++;
		x = start + *time +30;
		while (!(T0IR)) {
			if (T0TC > x)
				return (l + lowpulse);	
		}
		T0IR = 0xff;
		x = T0CR3;
		lowpulse = x-l;
	}
}

unsigned char decodewide(unsigned long value, struct widetable_* wide, unsigned char hi) {
	
	unsigned char x;
	unsigned long minvalue;
	unsigned long maxvalue;
	unsigned long dif;
	
	dif = (value / 10);
	
	if (dif < 2)
		dif = 2;
	if (value > 2)
		minvalue = value - dif;
	else
		minvalue = 0;
		
	maxvalue = value + dif;
	
	for (x=0;x < (wide->count);x++) {
		if ((wide->value[x] >= minvalue) && (wide->value[x] <= maxvalue)) {
			if (hi)
				wide->value[x] = value;
			return x;
		}
	}
	if (wide->count < 10) {
		wide->value[wide->count] = value;
		wide->count++;
		return (wide->count -1);
	} 
	else {
		return 0x0f;
	}
}

void getIR(struct RAWcmd_* cmd) {

	unsigned char count;
	unsigned char x;
	unsigned long hightime, lowtime;
	unsigned int highpulse, lowpulse;
	unsigned long T0count;
	unsigned long pulsetime;
	unsigned long T1count;
	unsigned long l;
	unsigned long first;
	unsigned long firsth;
	
	struct widetable_ widetable;
	
	setSpeed(SPEED_60);
	
	disableIRQ();
	T0MCR = 0x00;
	FIOSET0 |= (1<<4);
	FIOCLR0 |= (1<<12);
	PINSEL1 |= (1<<27);
	T0CCR = (1<<9) | (1<<10) | (1<<11);
	
	lcd_fill(0);
	
	widetable.count = 0;
	count = 0;
	x=1;
		
	draw_string(0,0,"ir read start", 3, DRAW_PUT);
	
	T0IR = 0xff;
	while (!T0IR);
	T0TC = 0;
	T0IR = 0xff;
	T0count = waitIrHi(&highpulse,&hightime,0);
	firsth = hightime;
	pulsetime = (hightime * 1000) / highpulse;
	cmd->data[0] = ((decodewide(highpulse, &widetable,1) & 0x0F)<<4);
	
	
	while (!(T0IR));
	first = T0CR3;
	T0IR = 0xff;
	T1count= T0CR3;
	while ((count < 99) && x) {
		lowtime = T1count-T0count;
		T0count = waitIrHi(&highpulse,&hightime,T1count);
		
		lowpulse = (lowtime *1000) / pulsetime;
		cmd->data[count] |= decodewide(lowpulse, &widetable,0);
		cmd->data[count+1] = decodewide(highpulse, &widetable,1)<<4;
		
		l=T0count + 500000;
		while (!(T0IR) && x) {
			if (T0TC > l)
				x=0;
		};
		T0IR = 0xff;
		T1count= T0CR3;
		count++;
	}
	if (x)
		count--;
	
	T0TC = 0;
	T0MCR = 0x03;
	FIOSET0 |= (1<<12);
	T0CCR = 0;
	T0IR = 0xff;
	enableIRQ();
	setBacklight(BL_AUTO);
	
	lcd_fill(0);
	for (x=0;(x<=count) && (x <= 100);x++) {
		draw_hexC((x%5)*15,8*(x/5),cmd->data[x], 3, DRAW_PUT);
		
	}

	draw_numU(80,2,first,0, 3, DRAW_PUT);
	draw_numU(80,10,firsth,0, 3, DRAW_PUT);
	draw_numU(80,18,pulsetime,0, 3, DRAW_PUT);
	
	for (x=0;x<widetable.count;x++) {
		draw_numU(80,36+(x*8),widetable.value[x],0, 3, DRAW_PUT);
		cmd->pulselen[x] = widetable.value[x];
	}
	
	cmd->hi_border = 1;
	cmd->lo_border = 4;
	cmd->tval = (15 * pulsetime) / (cmd->lo_border * 1000);
	cmd->len = count+1;
	
	draw_numU(80,26,cmd->tval,0, 3, DRAW_PUT);

	setSpeed(SPEED_30);
	
}
