/*
    ir_raw.c - raw ir encoder
    Copyright (C) 2008  <telekatz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "lpc2220.h"
#include "infrared.h"
#include "ir_raw.h"
#include "encoders.h"
#include "codes.h"

extern volatile unsigned char mod_enable;
extern volatile unsigned int cycles;
extern volatile unsigned long keyMap[42];

extern unsigned int	ir_cmd;
extern unsigned int	ir_actcmd;
extern unsigned int	ir_startstop;
extern unsigned int	ir_state;
extern unsigned int	ir_bit;
extern unsigned int	ir_numbits;
extern unsigned int	ir_wait;
extern unsigned int	ir_pause;
extern unsigned int	ir_leadtime;
extern unsigned int	ir_bittime0;
extern unsigned int	ir_bittime1;
extern unsigned int	ir_toggle;
extern unsigned int	ir_len;
extern unsigned int	ir_trail;
extern unsigned int	ir_preamble;

extern char sysInfo;

unsigned char* raw_cmd;
unsigned int* raw_bitcycles;

#define RAW_IDLE	0x00
//#define RAW_START_H	0x01
//#define RAW_START_L	0x02
#define RAW_HI	0x01
#define RAW_LO	0x02
//#define RAW_WAIT	0x05

struct RAWcmd_ test_cmd;

const struct irModule RAW_Module =
{ 
	RAW_Encode,
	RAW_Send,
	RAW_Repeat,
	RAW_Stop,
	76,	// carrier 
	91,	// carrier 
	1,	// carrier-on
	4,	// carrier-off
};

void __attribute__ ((section(".text.fastcode"))) RAW_Encode (void)
{
	switch(ir_state)
	{
		case RAW_IDLE:
			mod_enable = 0;
			break;

		case RAW_HI:
			mod_enable = 1;
			cycles = raw_bitcycles[raw_cmd[ir_bit]>>4];
			ir_state++;
			break;

		case RAW_LO:
			mod_enable = 0;
			cycles = raw_bitcycles[raw_cmd[ir_bit] & 0x0f];
			ir_bit++;
			if (ir_bit >= ir_numbits)
				ir_state = RAW_IDLE;
			else
				ir_state--;
			break;


	}
}

void RAW_Init(unsigned char map)
{
	/*if(map < RAW.num_tables)
	{
		ir_cmd	= 0x00000000;
		ir_actcmd	= 0x00000000;
		ir_bit	= 0;
		ir_numbits	= 12;
		ir_wait	= 0;
		ir_pause	= RAW_GTIME;
		ir_state	= RAW_IDLE;
		copyMapS((unsigned short*)RAW.table[map].codes);
		setIR(RAW_Module);
	}*/
}

void RAW_Send(unsigned long cmd)
{
	raw_cmd = &(test_cmd.data[0]);
	raw_bitcycles = &(test_cmd.pulselen[0]);
	setIR(RAW_Module);
	T1MR0 = test_cmd.tval;
	ir_numbits = test_cmd.len;
	RAW_Repeat();
}

void RAW_Repeat(void)
{
	if(ir_state == RAW_IDLE)
	{
		ir_bit	= 0;
		ir_state++;
		runIR();
	}

}

void RAW_Stop(void)
{

waitend:
		if(ir_state != RAW_IDLE)
			goto waitend;
	stopIR();
}
